#ifndef VKF_ATTRIBUTES_HPP
#define VKF_ATTRIBUTES_HPP

#define DEFAULT_HOST "localhost"
#define DEFAULT_PASS "toor"
#define DEFAULT_USER "root"

#include <mariadb++/account.hpp>
#include <mariadb++/connection.hpp>

#include <string>

namespace vkf {

class CAttributes {
    public:
    /** default constructor */
    CAttributes() = delete;
    CAttributes(const std::string &verges_table_name,
                const std::string &database,
                const std::string &url = DEFAULT_HOST,
                const std::string &user = DEFAULT_USER,
                const std::string &password = DEFAULT_PASS
                ) {
        // set up the account
        mariadb::account_ref acc = mariadb::account::create(url, user, password);
        // create connection
        mariadb::connection_ref con = mariadb::connection::create(acc);
        // create the database, if it does not exist
        con->execute("CREATE DATABASE IF NOT EXISTS " + database + " DEFAULT CHARACTER SET utf8");
        con->execute("USE " + database);
        // destroy the Table encoder, if it exists
        con->execute("DROP TABLE IF EXISTS " + verges_table_name);
        // costruct relational structute of the Table encoder
        std::string sql_query("CREATE TABLE IF NOT EXISTS " + verges_table_name);
        sql_query.append(" (row_id int(8) NOT NULL AUTO_INCREMENT, ");
        sql_query.append("attribute_name varchar(255) NOT NULL DEFAULT '', ");
        sql_query.append("value_verge float(53) NOT NULL DEFAULT 0.E1, ");
        sql_query.append("PRIMARY KEY(row_id)) ");
        sql_query.append("ENGINE = InnoDB");
        con->execute(sql_query);
        con->~connection();
        acc->~account();
    }
    ~CAttributes() = default;
};

class DAttributes {
    public:
    /** default constructor */
    DAttributes() = delete;
    DAttributes(const std::string &encoder_table_name,
                const std::string &database,
                const std::string &url = DEFAULT_HOST,
                const std::string &user = DEFAULT_USER,
                const std::string &password = DEFAULT_PASS
                ) {
        // set up the account
        mariadb::account_ref acc = mariadb::account::create(url, user, password);
        // create connection
        mariadb::connection_ref con = mariadb::connection::create(acc);
        // create the database, if it does not exist
        con->execute("CREATE DATABASE IF NOT EXISTS " + database + " DEFAULT CHARACTER SET utf8");
        con->execute("USE " + database);
        // destroy the Table encoder, if it exists
        con->execute("DROP TABLE IF EXISTS " + encoder_table_name);
        // costruct relational structute of the Table encoder
        std::string sql_query("CREATE TABLE IF NOT EXISTS " + encoder_table_name);
        sql_query.append(" (row_id int(8) NOT NULL AUTO_INCREMENT, ");
        sql_query.append("attribute_name varchar(255) NOT NULL DEFAULT '', ");
        sql_query.append("attribute_value varchar(255) NOT NULL DEFAULT 'null', ");
        sql_query.append("value_letter char(1) NOT NULL DEFAULT '_', ");
        sql_query.append("value_bitset int(16) NOT NULL DEFAULT 0, ");
        sql_query.append("bitset_length int(8) NOT NULL DEFAULT 1, ");
        sql_query.append("PRIMARY KEY(row_id)) ");
        sql_query.append("ENGINE = InnoDB");
        con->execute(sql_query);
        con->~connection();
        acc->~account();
    }
    ~DAttributes() = default;
};

} // namespace vkf
#endif // VKF_ATTRIBUTES_HPP
