#ifndef VKF_BEGET_HPP
#define VKF_BEGET_HPP

#define DEFAULT_HOST "localhost"
#define DEFAULT_PASS "toor"
#define DEFAULT_USER "root"

#include <mariadb++/account.hpp>
#include <mariadb++/connection.hpp>

#include <vector>
#include <cassert>
#include <cmath>
#include <string>
#include <sstream>
#include <iostream>
#include <fstream>

//#include "Join.hpp"

namespace vkf {

class Beget : public std::map<std::pair<std::string,std::string>, std::pair<double,std::pair<double,double>>> {
  public:
    Beget(  const std::string &complex_table_name,
            const std::string &database,
            const std::string &db_host = DEFAULT_HOST,
            const std::string &db_user = DEFAULT_USER,
            const std::string &db_pswd = DEFAULT_PASS
        ) {
        // set up the account
        mariadb::account_ref acc = mariadb::account::create(db_host, db_user, db_pswd);
        // create connection
        mariadb::connection_ref con = mariadb::connection::create(acc);
        con->execute("USE " + database);
        // obtain content of table encoder
        mariadb::result_set_ref result = con->query("SELECT * FROM " + complex_table_name);
        con->~connection();
        acc->~account();
        // reading from a result set
        while (result->next()) {
            this->emplace(std::make_pair(result->get_string("first_name"),result->get_string("second_name")),
                std::make_pair(result->get_double("bias"),
                    std::make_pair(result->get_double("first_coeff"), result->get_double("first_coeff"))));
        }
    };

    ~Beget() = default;

    double Estimate(double value1, double value2, const std::string name1, const std::string name2) const {
        std::pair<double,std::pair<double,double>> weights = this->find(std::pair<std::string,std::string>(name1,name2))->second;
        return weights.first + weights.second.first*value1 + weights.second.second*value2;
    };

};

} // namespace vkf

#endif // VKF_BEGET_HPP

