#include "Candidate.hpp" // paired header file

/**< AOMatrix constructor from Sample implementation */
vkf::AOMatrix::AOMatrix(const Sample& rhs, bool is_negative) {
    Sample::const_iterator it = rhs.cbegin();
    const std::size_t row_length = (*it)->size();
//    std::cout << "row_length is " << row_length << std::endl;
    for (std::size_t ind = 0; ind < row_length; ++ind)
        m_Columns.push_back(ParentsPtr(new Parents()));
    for (; it != rhs.cend(); ++it) {
        if ((*it)->GetSign() != is_negative) {
            m_Rows.push_back(FragmentPtr(new Fragment((boost::dynamic_bitset<>)*(*it))));
//            std::cout << *m_Rows.back() << std::endl;
            for (std::size_t ind = 0; ind < row_length; ++ind)
                m_Columns[ind]->push_back((bool)((boost::dynamic_bitset<>) *(*it))[ind]);
        }
    }
    const std::size_t column_length = m_Rows.size();
//    std::cout << "column_length is " << column_length << std::endl;
    std::uniform_int_distribution<unsigned long long>::param_type new_range(0, column_length + row_length - 1);
    index_distribution.param(new_range);

    /**< The private members of class Candidate can be accessed because AOMatrix is declared as a friend class of Candidate */
    m_BottomCandidate.m_Attributes.set(); // Sets every bit in this bitset to 1
    m_BottomCandidate.m_Attributes.resize(row_length, true) ; // Changes the number of bits of the bitset to row_length. 
    for (boost::dynamic_bitset<>::size_type indx = 0; indx < column_length; ++indx) {
        m_BottomCandidate.m_Attributes &= *m_Rows[indx];
    }
    m_BottomCandidate.m_Objects.set(); // Sets every bit in this bitset to 1
    m_BottomCandidate.m_Objects.resize(column_length, true); // Changes the number of bits of the bitset to column_length.
//    std::cout << "m_BottomCandidate.m_Attributes is " << m_BottomCandidate.m_Attributes << std::endl;
//    std::cout << "m_BottomCandidate.m_Objects is " << m_BottomCandidate.m_Objects << std::endl;
    m_TopCandidate.m_Objects.set(); // Sets every bit in this bitset to 1
    m_TopCandidate.m_Objects.resize(column_length, true); // Changes the number of bits of the bitset to column_length.
    for (boost::dynamic_bitset<>::size_type indx = 0; indx < row_length; ++indx) {
        m_TopCandidate.m_Objects &= *m_Columns[indx];
    }
    m_TopCandidate.m_Attributes.set(); // Sets every bit in this bitset to 1
    m_TopCandidate.m_Attributes.resize(row_length, true); // Changes the number of bits of the bitset to row_length.
//    std::cout << "m_TopCandidate.m_Attributes is " << m_TopCandidate.m_Attributes << std::endl;
//    std::cout << "m_TopCandidate.m_Objects is " << m_TopCandidate.m_Objects << std::endl;
};

/**< AOMatrix destructor implementation */
vkf::AOMatrix::~AOMatrix() {
    while(!m_Rows.empty())
    {
        m_Rows.pop_back();
    }
    while(!m_Columns.empty())
    {
        m_Columns.pop_back();
    }
}

// Derive set of attributes corresponding to given set of objects
vkf::Fragment& vkf::AOMatrix::Derive(const vkf::Parents& objects, vkf::Fragment& result) const {
    std::size_t index = objects.find_first();
    result.set();
    if (index == boost::dynamic_bitset<>::npos)
        return result;
    else
        result &= *m_Rows[index];
    while(index != boost::dynamic_bitset<>::npos)
    {
        index = objects.find_next(index);
        if (index != boost::dynamic_bitset<>::npos)
            result &= *m_Rows[index];
    }
    return result;
}

// Derive set of objects corresponding to given set of attributes
vkf::Parents& vkf::AOMatrix::Derive(const vkf::Fragment& attributes, vkf::Parents& result) const {
    std::size_t index = attributes.find_first();
    result.set();
    if (index == boost::dynamic_bitset<>::npos)
        return result;
    else
        result &= *m_Columns[index];
    while (index != boost::dynamic_bitset<>::npos)
    {
        index = attributes.find_next(index);
        if (index != boost::dynamic_bitset<>::npos)
            result &= *m_Columns[index];
    }
    return result;
}

/** \brief Extreme Candidate constructor implementation
 * \param the first parameter fContext of type AOMatrix contains the bottom and the top candidates
 * \param the second parametr bType determines what type extreme candidate to copy into new candidate */
vkf::Candidate::Candidate(const vkf::AOMatrix& context, const vkf::CandidateType& bType) {
    if (bType == vkf::CandidateType::TOP_CANDIDATE)
    {
        // The private members of class AOMatrix can be accessed because Candidate is declared as a friend class of AOMatrix
        m_Attributes = context.m_TopCandidate.m_Attributes;
        m_Objects = context.m_TopCandidate.m_Objects;
    }
    else // bType == CandidateType::BOTTOM_CANDIDATE
    {
        m_Attributes = context.m_BottomCandidate.m_Attributes;
        m_Objects = context.m_BottomCandidate.m_Objects;
    }
}

/** \brief Complete finishes Candidate computation
 * \param context contains the formal context with training examples 
 * \param previous_move_up 
 * \return second parameter */
bool vkf::Candidate::Complete(const vkf::AOMatrix& context, bool previous_move_up)
{
    if (previous_move_up)
        context.Derive(m_Objects, m_Attributes); /**< Fragment& AOMatrix::Derive(const Parents& objects, Fragment& result) */
    else // previous_move_up == false
        context.Derive(m_Attributes, m_Objects); /**< Parents& AOMatrix::Derive(const Fragment& attributes, Parents& result) */
    return(previous_move_up);
}
