#ifndef VKF_GENERATOR_HPP
#define VKF_GENERATOR_HPP

#include <mariadb++/account.hpp>
#include <mariadb++/connection.hpp>

#include "Inhabit.hpp"
#include "Verges.hpp"

namespace vkf {

class Generator : public std::vector<std::pair<Verges,std::string>> {
    public:
        ~Generator() = default;

        Generator(  const std::string &complex_table_name,
                    const std::string &data_table_name,
                    const std::string &verges_table_name,
                    const std::string &database,
                    std::uint32_t real_verges_number = 0,
                    std::uint32_t phantom_verges_number = 1,
                    const std::string &db_host = DEFAULT_HOST,
                    const std::string &db_user = DEFAULT_USER,
                    const std::string &db_pswd = DEFAULT_PASS
                ) {
            const vkf::Join join(data_table_name, database, db_host, db_user, db_pswd);
            std::uint32_t length;
                if (real_verges_number == 0) {
                    length = std::log2(join.size());
                } else {
                    length = real_verges_number;
                }
            std::uint32_t attr_total = join.GetAttrTotal();
            for (std::uint32_t ndx = 0; ndx < attr_total; ndx++) {
                std::string attr_name = join.GetAttrName(ndx);
                Inhabit axe(join, ndx);
                Verges temp(axe);
                temp.Populate(length);
                this->emplace_back(temp, attr_name);
            }
            // set up the account
            mariadb::account_ref acc = mariadb::account::create(db_host, db_user, db_pswd);
            // create connection
            mariadb::connection_ref con = mariadb::connection::create(acc);
            con->execute("USE " + database);
            // obtain content of table encoder
            mariadb::result_set_ref result = con->query("SELECT * FROM " + complex_table_name);
            // reading from a result set
            std::uint32_t attr1_index = 0;
            std::uint32_t attr2_index = 0;
            while (result->next()) {
                std::string attr_name = result->get_string("first_name");
                std::string new_attr_name = attr_name;
                auto search = join.GetAttrIndex(attr_name);
                if (search != join.GetAttrTotal()) attr1_index = search;
                else break;
                new_attr_name.append("__");
                attr_name = result->get_string("second_name");
                new_attr_name.append(attr_name);
                search = join.GetAttrIndex(attr_name);
                if (search != join.GetAttrTotal()) attr2_index = search;
                else break;
//                std::cout << "name of new attribute: " << new_attr_name << std::endl;
                double bias = result->get_double("bias");
                double intercept1 = result->get_double("first_coeff");
                double intercept2 = result->get_double("second_coeff");
                Inhabit axe;
                std::uint32_t sample_size = join.size();
                for (std::uint32_t ndx = 0; ndx < sample_size; ndx++) {
                    axe.emplace_back(bias + join[ndx][attr1_index]*intercept1 + join[ndx][attr2_index]*intercept2, join[ndx].IsPlus());
                }
                axe.sort(vkf::pair_compare);
                Verges temp(axe);
                temp.Populate(phantom_verges_number);
                this->emplace_back(temp, new_attr_name);
            }
            // create table verges
            con->execute("DROP TABLE IF EXISTS " + verges_table_name);
            std::string sql_query("CREATE TABLE IF NOT EXISTS " + verges_table_name);
            sql_query.append(" (row_id int(8) NOT NULL AUTO_INCREMENT, ");
            sql_query.append("attribute_name varchar(255) NOT NULL DEFAULT 'null', ");
            sql_query.append("value_verge float(53) NOT NULL DEFAULT 0E0, ");
            sql_query.append("PRIMARY KEY(row_id)) ");
            sql_query.append("ENGINE = InnoDB");
            con->execute(sql_query);
            mariadb::statement_ref stmt = con->create_statement("INSERT INTO " + verges_table_name + " VALUES (NULL, ?, ?)");
            for (auto it = this->begin(); it != this->end(); it++) {
                std::string attribute_name = it->second;
                stmt->set_string(0, attribute_name); // attribute_name
                stmt->set_double(1, it->first.GetMinimum()); // value_minimum
                stmt->insert();
                for (auto verge : it->first) {
                    stmt->set_double(1, verge); // value_verge
                    stmt->insert();
                }
                stmt->set_double(1, it->first.GetMaximum()); // value_maximum
                stmt->insert();
//                std::cout << "saved verges for  " << attribute_name << std::endl;
            }
//            stmt->~statement();
            con->~connection();
            acc->~account();
        };
};

} // namespace vkf

#endif // VKF_GENERATOR_HPP
