#ifndef VKF_JOIN_HPP
#define VKF_JOIN_HPP

#define DEFAULT_HOST "localhost"
#define DEFAULT_PASS "toor"
#define DEFAULT_USER "root"

#include <mariadb++/account.hpp>
#include <mariadb++/connection.hpp>

#include <map>
#include <cassert>
#include <cmath>
#include <string>
#include <sstream>
#include <iostream>
#include <fstream>

namespace vkf {

class Description : public std::vector<double> {
    private:
        bool sign;
    public:
        ~Description() = default;
        Description(bool sign) : sign(sign) {};
        bool IsPlus() const { return sign; };
};

class Join : public std::vector<Description> {
    private:
//        std::map<std::uint32_t,std::string> attributes_names;
        std::map<std::string,std::uint32_t> attributes_names;
    public:
//        std::string GetAttrName(std::uint32_t attr_ndx) const {
//            auto search = attributes_names.find(attr_ndx);
//            if (search != attributes_names.end()) {
//                return search->second;
//            } else {
//                return std::string("_not_found");
//            }
//        };
        std::string GetAttrName(std::uint32_t attr_index) const {
            auto it = std::find_if(attributes_names.begin(), attributes_names.end(),
                [&](const std::pair<std::string,std::uint32_t> &arg){ return arg.second == attr_index; });
            if (it != attributes_names.end()) {
                return it->first;
            } else {
                return std::string("_not_found");
            }
        };
//        std::uint32_t GetAttrIndex (std::string attr_name) {
//            auto it = std::find_if(attributes_names.begin(), attributes_names.end(),
//                [&](const std::pair<std::uint32_t,std::string> &arg){ return arg.second == attr_name; });
//            if (it != attributes_names.end()) {
//                return it->first;
//            } else {
//                return attributes_names.size();
//            }
//        };
        std::uint32_t GetAttrIndex (std::string attr_name) const {
            auto search = attributes_names.find(attr_name);
            if (search != attributes_names.end()) {
                return search->second;
            } else {
                return attributes_names.size();
            }
        };

        std::uint32_t GetAttrTotal() const { return attributes_names.size(); };

        ~Join() = default;
        Join(   const std::string &data_table_name,
                const std::string &database,
                const std::string &db_host = DEFAULT_HOST,
                const std::string &db_user = DEFAULT_USER,
                const std::string &db_pswd = DEFAULT_PASS
            ) {
            // set up the account
            mariadb::account_ref acc = mariadb::account::create(db_host, db_user, db_pswd);
            // create connection
            mariadb::connection_ref con = mariadb::connection::create(acc);
            con->execute("USE " + database);
            // obtain content of table encoder
            mariadb::result_set_ref result = con->query("SELECT * FROM " + data_table_name);
            con->~connection();
            acc->~account();
            // reading from a result set
            std::uint32_t attr_index = 0;
            for (std::uint32_t ndx = 0; ndx < result->column_count(); ndx++) {
                if (result->column_type(ndx) == mariadb::value::double64) { // mariadbpp::value enum type
//                    attributes_names.emplace(attr_index, result->column_name(ndx));
                    attributes_names.emplace(result->column_name(ndx), attr_index);
                    attr_index++;
                }
            }
            while (result->next()) {
                this->emplace_back(result->get_boolean("real_sign"));
                for (std::uint32_t ndx = 0; ndx < result->column_count(); ndx++) {
                    if (result->column_type(ndx) == mariadb::value::double64) { // mariadbpp::value enum type
                        this->back().push_back(result->get_double(ndx));
                    }
                }
            }
        };

        void ComputeSave(   const std::string &complex_table_name,
                            const std::string &database,
                            const std::string &db_host,
                            const std::string &db_user,
                            const std::string &db_pswd
                        ); // see Join.cpp for implementation

};

class SemiRegress {
  private:
    std::uint32_t position;
    double w0;
    double w1;
  public:
    double GetBias() const { return w0; };
    double GetCoeff() const { return w1; };

	double Predict(const Join &sample, const std::uint32_t ndx) const{
        return 1.0/(1.0 + std::exp(-w0 - w1*sample[ndx][position]));
    };

	double Optimize(const Join &sample, const double threshold); // see regress.cpp for implementation

	SemiRegress(std::uint32_t _pos) : position(_pos) {
        w0 = 0.0;
        w1 = 0.0;
    };

	~SemiRegress() = default;
};

class Regress {
  private:
    std::uint32_t pos1;
    std::uint32_t pos2;
    double w0;
    double w1;
    double w2;
  public:
    double GetBias() const { return w0; };
    double GetFirstCoeff() const { return w1; };
    double GetSecondCoeff() const { return w2; };

	double Predict(const Join &sample, const std::uint32_t ndx) const{
        return 1.0/(1.0 + std::exp(-w0 - w1*sample[ndx][pos1] - w2*sample[ndx][pos2]));
    };

	double Optimize(const Join &sample, const double threshold); // see regress.cpp for implementation

    double Estimate(const Join &sample, const std::uint32_t ndx) const{
        return w0 + w1*sample[ndx][pos1] + w2*sample[ndx][pos2];
    };

	Regress(std::uint32_t fpos, std::uint32_t spos) : pos1(fpos), pos2(spos) {
        w0 = 0.0;
        w1 = 0.0;
        w2 = 0.0;
    };

	~Regress() = default;
};

} // namespace vkf

#endif // VKF_JOIN_HPP
