#ifndef VKF_LATTICES_HPP
#define VKF_LATTICES_HPP

#define DEFAULT_DB "mushrooms_db"
#define DEFAULT_HOST "localhost"
#define DEFAULT_PASS "toor"
#define DEFAULT_USER "root"

#include <mariadb++/account.hpp>
#include <mariadb++/connection.hpp>

#include <string>

namespace vkf {
class Lattices {
    public:
    /** default constructor */
    Lattices() = delete;
    Lattices(   const std::string &lattices_table_name,
                const std::string &database = DEFAULT_DB,
                const std::string &url = DEFAULT_HOST,
                const std::string &user = DEFAULT_USER,
                const std::string &password = DEFAULT_PASS
                ) {
        // set up the account
        mariadb::account_ref acc = mariadb::account::create(url, user, password);
        // create connection
        mariadb::connection_ref con = mariadb::connection::create(acc);
        // create the database, if it does not exist
        con->execute("CREATE DATABASE IF NOT EXISTS " + database + " DEFAULT CHARACTER SET utf8");
        con->execute("USE " + database);
        // destroy the Table orderings, if it exists
        con->execute("DROP TABLE IF EXISTS " + lattices_table_name);
        // costruct relational structute of the Table orderings
        std::string sql_query;
        sql_query.append("CREATE TABLE IF NOT EXISTS " + lattices_table_name);
        sql_query.append(" (row_id int(8) NOT NULL AUTO_INCREMENT, ");
        sql_query.append("attribute_name varchar(255) NOT NULL DEFAULT '', ");
        sql_query.append("source_value varchar(255) NOT NULL DEFAULT '', ");
        sql_query.append("target_value varchar(255) NOT NULL DEFAULT 'null', ");
        sql_query.append("PRIMARY KEY(row_id)) ");
        sql_query.append("ENGINE = InnoDB");
        con->execute(sql_query);
        con->~connection();
        acc->~account();
    }
    ~Lattices() = default;
};
} // namespace vkf
#endif // VKF_LATTICES_HPP
