#ifndef VKF_OBSTACLES_HPP
#define VKF_OBSTACLES_HPP
// Declare a class list of obstacles to be VKF-hypothesis.
// This list contains negative examples and artificial fragments whose subsets are declared as trivial similarities.

#include "Sample.hpp" // includes boostdll.h, general.h, and Encoded.hpp too

namespace vkf {
    typedef std::vector<FragmentPtr> ExamplesList;

    class Obstacles : public ExamplesList {
        public:
            /** default constructor */
            Obstacles() = default; // for Induction(Hypotheses&& rhs) constructor
//            Obstacles() = delete; // error: Induction(Hypotheses&& rhs) constructor uses deleted function
            /** copy-constructor */
            Obstacles(const Obstacles&) = delete;
            /** move-constructor */
            Obstacles(Obstacles&&) = default;
            /** copy-operator= */
            Obstacles& operator=(const Obstacles& rhs) = delete;
            /** move-operator= */
            Obstacles& operator=(Obstacles&& rhs) = default;
            /** Constructor from constant Sample lvalue-object declaration */
            Obstacles(const Sample& rhs, const bool is_negative) {
                for (Sample::const_iterator it = rhs.cbegin(); it != rhs.cend(); ++it) {
                    if ((*it)->GetSign() == is_negative) {
                        push_back(vkf::FragmentPtr(new Fragment((boost::dynamic_bitset<>)*(*it))));
//                        std::cout << (boost::dynamic_bitset<>)*(*it) << std::endl;
                    }
                }
//                std::cout << size() << std::endl;
            };
            /** Default destructor */
            ~Obstacles(){
                while(!empty()){
                    pop_back();
               }
            };
    };
} //namespace vkf
#endif // VKF_OBSTACLES_HPP
